import logging
import os
import platform
import sys

from dotenv import load_dotenv
from docopt import ParsedOptions


from cli.constants import LOG_DIR, LOG_FILE, CONFIG_PATH
from cli.models.config import ConfigObject


def init(parsed_options: ParsedOptions) -> ConfigObject:
    CONFIG_PATH_PREFIX = os.getenv('CONFIG_PATH_PREFIX') or ""
    cfg = ConfigObject(parsed_options, config_file_path=CONFIG_PATH_PREFIX + CONFIG_PATH)

    if cfg.app_inputs["debug"]:
        log_level = logging.DEBUG
        os.makedirs(LOG_DIR, exist_ok=True)
        logging.basicConfig(filename=os.path.join(LOG_DIR, LOG_FILE),
                            format='%(asctime)s - %(levelname)s:%(name)s - %(message)s',
                            level=log_level)
    else:
        logging.basicConfig(level=logging.CRITICAL + 1)
    logger = logging.getLogger(__name__)
    logger.info(f"{'=' * 200}")
    logger.info(f"Starting executing SE CLI Command Sequencer with the following inputs: {sys.argv[1:]}")
    logger.debug(f"{get_system_info()}")
    return cfg


def get_system_info():
    return {'OS': platform.system(), 'OS Version': platform.version(), 'Machine': platform.machine()}


def load_env():
    if os.path.exists("constants.env"):
        load_dotenv(dotenv_path="constants.env")



PACKAGE_NAME = "cast-ai-se-cli"

DEMO_OFF_CRONJOB = "hibernate-pause"
# DEMO_ON_CRONJOB = "hibernate-resume"

# DEBUG LOGS
LOG_DIR = "logs"
LOG_FILE = "all_logs.log"

CONFIG_PATH = "config.json"
SUPPORTED_COMMANDS = ["demo", "snapshot", "audit", "version"]

EPHEMERAL_NODE_STATUSES = ["deleted", "deleting", "draining", "cordoned"]
SUPPORTED_K8S = ['EKS', 'AKS', 'GKE']



import requests
from typing import Optional
from packaging import version
import os
from pkg_resources import get_distribution

from cli.constants import PACKAGE_NAME


def get_current_version() -> str:
    current_file_path = os.path.abspath(__file__)  # Get absolute path of the current file
    grandparent_dir_path = os.path.dirname(os.path.dirname(os.path.dirname(current_file_path)))  # Go 3 levels up
    version_file_path = os.path.join(grandparent_dir_path, 'version.txt')  # Construct the full path

    if os.path.exists(version_file_path):
        with open(version_file_path, 'r') as file:
            pkg_version = file.read().strip()
    else:
        try:
            pkg_version = get_distribution(PACKAGE_NAME).version
        except:
            return ""
    return pkg_version


def get_latest_version(package_name: str = PACKAGE_NAME) -> Optional[str]:
    try:
        response = requests.get(f'https://pypi.org/pypi/{package_name}/json')
        response.raise_for_status()
        data = response.json()
        latest_version = data['info']['version']
        return latest_version
    except requests.RequestException as e:
        print(f"Error: {e}")
        return None


def check_if_latest_version():
    l_version = get_latest_version()
    c_version = get_current_version()
    if not l_version or not c_version:
        return
    latest_version = version.parse(l_version)
    current_version = version.parse(c_version)
    if latest_version > current_version:
        print(f"New version ({latest_version}) is out! Please consider upgrading.\n")
    else:
        print(f"You are using latest version ({latest_version})...Respect!")



from cast_ai.se.contollers.snapshot_controller import SnapshotController
from cast_ai.se.models.misc_data_models import ReportDetailLevel
from cast_ai.se.services.snapshot_analysis_svc import SnapshotAnalyzer

from cli.models.config import ConfigObject
from cli.orchestrators.base_orch import BaseOrchestrator


class SnapshotOrchestrator(BaseOrchestrator):
    def __init__(self, cfg: ConfigObject):
        super().__init__(cfg)
        self._snapshot_ctrl = SnapshotController(cast_api_key=cfg.app_config["CAST"]["CASTAI_API_TOKEN"],
                                                 default_cluster_id=cfg.cid,
                                                 json_key_path=cfg.app_config["GKE"]["JSON_KEY_PATH"])
        self._snapshot_analyzer = SnapshotAnalyzer(self._snapshot_ctrl.raw_snapshot)
        self._snapshot_subcommand_mapping = {
            "brief": self.generate_brief_report,
            "detailed": self.generate_detailed_report,
            "extra": self.generate_extra_detailed_report,
            "csv": self.generate_csv_detailed_report,
        }

    def execute(self) -> None:
        subcommand = self._cfg.app_inputs["snapshot_subcommand"]
        if subcommand in self._snapshot_subcommand_mapping:
            self._snapshot_subcommand_mapping[subcommand]()
        else:
            raise ValueError(f'Invalid option: {subcommand}')

    def generate_brief_report(self):
        print(self._snapshot_analyzer.get_refined_snapshot_report(detail_lvl=ReportDetailLevel.BASIC))

    def generate_detailed_report(self):
        print(self._snapshot_analyzer.get_refined_snapshot_report(detail_lvl=ReportDetailLevel.DETAILED))

    def generate_extra_detailed_report(self):
        print(self._snapshot_analyzer.get_refined_snapshot_report(detail_lvl=ReportDetailLevel.EXTRA))

    def generate_csv_detailed_report(self):
        self._snapshot_analyzer.get_refined_snapshot_csv_report()



"""
se-cli: The swiss army command-line for Cast.AI SE`s.

Usage:
    se-cli [-c <cluster_id>] [-d] demo <prep|off> <eks|aks|gcp>
    se-cli [-c <cluster_id>] [-d] [-n] snapshot <analyze> <basic|detailed|extra|csv>
    se-cli [-c <cluster_id>] [-d] audit <analyze>
    se-cli version

Options:
    -c, --cluster_id <cluster_id>  (Optional) Specify the cluster ID (will assume correct kubectl context)
    -d, --debug                    Enable debug logging
    -n, --no_cfg                   config.json will be ignored and will not be created - cluster_id should be provided

Commands:
    demo      Manage the demo environment
    snapshot  Manage snapshots
    audit     Manage audit logs
    version   Shows current version

Subcommands for "demo":
    on       Prep demo environment for demo (Get off hibernation)
    off      Hibernate demo environment
    refresh  Disable BinPacking and prep demo

Optional subcommands for "demo" (overrides -c, --cluster_id <cluster_id> picks cluster_id from config file of cloud)
    eks     Use EKS config cluster_id and context
    aks     Use AKS config cluster_id and context
    gcp     (not supported yet)

"""
import sys

from cli.orchestrators.demo_orch import DemoOrchestrator
from cli.orchestrators.snapshot_orch import SnapshotOrchestrator
from cli.orchestrators.audit_orch import AuditOrchestrator
from cli.services.general_responder_svc import GeneralResponder
from cli.services.misc_svc import init
from cli.services.version_svc import check_if_latest_version

from docopt import docopt


COMMANDS_TO_CLASS_MAPTABLE = {
    "demo": DemoOrchestrator,
    "snapshot": SnapshotOrchestrator,
    "audit": AuditOrchestrator,
    "version": GeneralResponder
}


def main():
    try:
        check_if_latest_version()
        parsed_options = docopt(__doc__, sys.argv[1:])
        cfg = init(parsed_options)
        orchestrator_class = COMMANDS_TO_CLASS_MAPTABLE[cfg.app_inputs["command"]]
        main_orch = orchestrator_class(cfg)
        main_orch.execute()
    except Exception as e:
        print(f"An error occurred: {e}")


if __name__ == '__main__':
    main()



import logging

from cli.models.config import ConfigObject
from cli.services.version_svc import get_current_version


class GeneralResponder:
    def __init__(self, cfg: ConfigObject):
        self._logger = logging.getLogger(__name__)
        self._cfg = cfg
        self._commands_mapping = {
            "version": self._print_current_version,
        }

    def execute(self) -> None:
        if self._cfg.app_inputs["command"] in self._commands_mapping:
            self._commands_mapping[self._cfg.app_inputs["command"]]()
        else:
            raise ValueError(f'Invalid option: {self._cfg.app_config["command"]}')

    @staticmethod
    def _print_current_version():
        version = get_current_version()
        if version:
            print(f"CAST.AI-SE-CLI version = {get_current_version()}")
        else:
            print("There was an issue figuring out the version of this amazing CLI tool")



import logging
import re

from cast_ai.se.contollers.kubectl_controller import KubectlController

from cli.config_constants import CONFIG_TEMPLATE

logger = logging.getLogger(__name__)


def is_config_file_valid(config: dict, original_config=CONFIG_TEMPLATE) -> bool:
    keys1 = set(config.keys())
    keys2 = set(original_config.keys())

    if keys1 != keys2:
        logger.error(f"Found mismatch in configuration file keys: {keys1}!={keys2}")
        return False

    for key in keys1:
        value1 = config[key]
        value2 = original_config[key]
        is_dict1 = isinstance(value1, dict)
        is_dict2 = isinstance(value2, dict)

        if is_dict1 != is_dict2:
            logger.error(f"Found mismatch in configuration file: {value1}!={value2}")
            return False

        if is_dict1 and is_dict2:
            if not is_config_file_valid(value1, value2):
                return False
    return True


def is_cluster_id(cluster_id: str) -> bool:
    regex = r"^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$"
    match = re.search(regex, cluster_id)
    logger.debug(f"is {cluster_id} a match for cluster_id format ? => {str(match)}")
    return bool(match)


def validate_kubectl_context(kube_ctrl: KubectlController, cloud: str) -> None:
    identified_cloud_context = kube_ctrl.get_context_cloud_type()
    if not identified_cloud_context == cloud:
        logger.warning(f"Kubectl context({identified_cloud_context}) doesn't match the one on CAST ({cloud})")



from datetime import datetime


class TimedText:
    def __init__(self, text):
        self.text = text
        self._start = datetime.now()

    def __str__(self):
        now = datetime.now()
        delta = now - self._start
        return f"{self.text} ({round(delta.total_seconds(), 1)}s)"
